import { getContentToBuild } from './getContnetToBuild.mjs';
import { getRemoteContentToBuild } from './getRemoteContentToBuild.mjs';
import { updateDetailsFileMapConfig2 } from './updateFileMapConfig.mjs';
import { createFile } from '../util/createFile.mjs';
import { createPath } from '../util/createPath.mjs';
import { debugFunction } from '../util/debugFunction.mjs';
import { isFileOrFolderExists } from '../util/isFileOrFolderExists.mjs';
import { parseJSON } from '../util/parseJSON.mjs';
import { readFile } from '../util/readFile.mjs';

const buildFromConfig = async (config) => {
    debugFunction(config.isDebug, 'start', '[INIT] buildFromConfig');
    let sumFileMapConfig = await readFile(config.sumFileMapConfig).then(async (bufferData) => parseJSON(bufferData.toString()));
    for (const realFilePath in sumFileMapConfig.sumFileMap) {
        // because we only check for default, the rest should be generated manually or only use getContent
        // const SUMKeySuffix = "defaultFile"
        for (const SUMKeySuffix in sumFileMapConfig.sumFileMap[realFilePath]) {
            if (SUMKeySuffix === '_') {
                continue;
            }
            const currentFileObject = sumFileMapConfig.sumFileMap[realFilePath][SUMKeySuffix];
            if (!(await isFileOrFolderExists({ isDebug: config.isDebug, filePath: currentFileObject.path }))) {
                await createFile({
                    filePath: createPath(currentFileObject.path),
                    content: '',
                    isDebug: config.isDebug,
                    options: {
                        overwriteFile: true,
                    },
                }).then(async () => {
                    sumFileMapConfig = await updateDetailsFileMapConfig2({
                        config,
                        operation: 'createSuffixFile',
                        realFilePath,
                        SUMKeySuffix: SUMKeySuffix,
                    });
                });
            }
            // There's no need for others than default to appear here
            if (sumFileMapConfig.fileMap.includes(currentFileObject.SUMSuffixFileName)) {
                const content = await getRemoteContentToBuild({
                    config,
                    sumObject: currentFileObject,
                });
                if (content) {
                    await createFile({
                        filePath: currentFileObject.path,
                        content,
                        options: {
                            overwriteFile: true,
                        },
                    });
                }
            }
        }
        // const updatedSumFileMapConfig: FileMapConfig = await readFile(config.sumFileMapConfig).then(async (bufferData) =>
        //   parseJSON(bufferData.toString())
        // );
        const realFileObject = sumFileMapConfig.sumFileMap[realFilePath]['_'];
        if (sumFileMapConfig.sumFileMap) {
            const sumSetObject = sumFileMapConfig.sumFileMap[realFilePath];
            const content = await getContentToBuild(sumSetObject);
            if (content) {
                await createFile({
                    filePath: realFileObject.path,
                    content,
                    isDebug: config.isDebug,
                }).then(async () => {
                    sumFileMapConfig = await updateDetailsFileMapConfig2({
                        config,
                        operation: 'createSUMRealFile',
                        realFilePath,
                    });
                });
            }
        }
    }
    debugFunction(config.isDebug, { sumFileMapConfig }, '[INIT] buildFromConfig');
    return { config, sumFileMapConfig };
};

export { buildFromConfig };
