import { cleanUpFiles } from './__tests__/cleanForTests.mjs';
import { mockConfig, mockSumFileMapConfig } from './__tests__/const.mjs';
import { searchFilesInDirectory } from './__tests__/searchFilesInDirectory.mjs';
import { prepareBaseSumFileMap } from './prepareBaseFile.mjs';
import { createFile } from '../util/createFile.mjs';

describe('prepareBaseSumFileMap', () => {
    let config;
    let sumFileMapConfig;
    beforeEach(async () => {
        config = { ...mockConfig.step.downloadConfigFile.forInit };
        sumFileMapConfig = { ...mockSumFileMapConfig.step.downloadConfigFile.forInit };
        await cleanUpFiles({
            sumCatalog: config.sumCatalog,
            directoryPath: config.projectCatalog,
            isDebug: config.isDebug,
        });
    });
    afterEach(async () => {
        await cleanUpFiles({
            sumCatalog: config.sumCatalog,
            directoryPath: config.projectCatalog,
            isDebug: config.isDebug,
        });
    });
    it('should return correct content', async () => {
        await createFile({
            filePath: config.sumConfigFilePath,
            content: JSON.stringify(config),
        });
        await createFile({
            filePath: config.sumFileMapConfig,
            content: JSON.stringify(sumFileMapConfig),
        });
        const result = await prepareBaseSumFileMap(config);
        const allFiles = await searchFilesInDirectory({
            directoryPath: config.projectCatalog,
            excludedFileNames: ['.DS_Store'],
            excludedPhrases: ['.backup'],
        });
        expect({ ...result, allFiles }).toStrictEqual({
            config: mockConfig.step.createConfigFile,
            sumFileMapConfig: mockSumFileMapConfig.step.prepareBaseSumFileMap,
            allFiles: ['./test/mockProject/.sum/repositoryMap.json', './test/mockProject/.sum.config.json'],
        });
    });
});
